/**
 * Copyright (C) 2010 Cloudfarming <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.db;

import javax.persistence.EntityManager;
import org.openide.util.Lookup;
import org.openide.util.LookupEvent;
import org.openide.util.LookupListener;

/**
 * The EntityManagerProvider functions as global provider and proxies the getEntityManager
 * method to any EntityManagerProviderSupport visible in the lookup.
 *
 * This way persisting modules do not need to know about where the entityManager is coming from.
 * 
 *
 * @author Timon Veenstra
 */
public class EntityManagerProvider implements EntityManagerProviderSupport {

    private static EntityManagerProvider instance;
    private EntityManagerProviderSupport providerSupport;

    public static EntityManagerProvider getDefault() {
        if (instance == null) {
            instance = new EntityManagerProvider();
        }
        return instance;
    }

    private EntityManagerProvider() {
        Lookup.Result<EntityManagerProviderSupport> result = Lookup.getDefault().lookupResult(EntityManagerProviderSupport.class);

        for (EntityManagerProviderSupport ps : result.allInstances()) {
            providerSupport = ps;
        }

        result.addLookupListener(new LookupListener() {

            @Override
            public void resultChanged(LookupEvent ev) {
                for (EntityManagerProviderSupport ps : ((Lookup.Result<EntityManagerProviderSupport>) ev.getSource()).allInstances()) {
                    providerSupport = ps;
                }
            }
        });

    }

    @Override
    public EntityManager getEntityManager(String moduleName) {
        if (providerSupport == null) {
            throw new IllegalStateException("No EntityManagerProviderSupport provided ");
        }
        return this.providerSupport.getEntityManager(moduleName);
    }
}
